unit Injector;

interface

uses Windows, SysUtils, Process;

function LoadLibrary(Process: TProcess; Path: String; InitializationName: PAnsiChar): Cardinal;

implementation

function GetErrorMessage(Error: DWORD): string;
begin
  case Error of
    20000: Result := 'Unable to find function to call!';

    else
      begin
        SetLength(Result, 65536 div SizeOf(Char));

        if FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nil, Error, 0, PChar(Result), Length(Result), nil) > 0 then
          Result := 'External error (' + IntToStr(Error) + ')'#13#10#13#10 + Result
        else
          Result := 'External error (' + IntToStr(Error) + ')';
      end;

  end;
end;

{ Load Library }

type
  PLoadLibraryInfo = ^TLoadLibraryInfo;
  TLoadLibraryInfo = record
    Result: Cardinal;
    LoadLibrary: function(lpLibFileName: PWideChar): HMODULE; stdcall;
    GetLastError: function: DWORD; stdcall;
    GetProcAddress: function(Module: Cardinal; Name: PAnsiChar): Pointer; stdcall;
    Path: PWideChar;
    InitializationName: PAnsiChar;
  end;

function ExternLoadLibrary(LoadLibraryInfo: PLoadLibraryInfo): DWORD; stdcall;
var
  Call: function: Cardinal; stdcall;
  Handle: THandle;
begin
  Handle := LoadLibraryInfo.LoadLibrary(LoadLibraryInfo.Path);
  if Handle = 0 then
    Exit(LoadLibraryInfo.GetLastError);

  if LoadLibraryInfo.InitializationName <> nil then
    begin
      @Call := LoadLibraryInfo.GetProcAddress(Handle, LoadLibraryInfo.InitializationName);

      if @Call = nil then
        Exit(20000)
      else
        LoadLibraryInfo.Result := Call;
    end;

  Result := 0;
end;

function LoadLibrary(Process: TProcess; Path: String; InitializationName: PAnsiChar): Cardinal;
var
  LoadLibraryInfo: TLoadLibraryInfo;
  Extern: PLoadLibraryInfo;
begin
  LoadLibraryInfo.LoadLibrary := GetProcAddress(GetModuleHandle('kernel32.dll'), 'LoadLibraryW');
  LoadLibraryInfo.GetLastError := GetProcAddress(GetModuleHandle('kernel32.dll'), 'GetLastError');
  LoadLibraryInfo.GetProcAddress := GetProcAddress(GetModuleHandle('kernel32.dll'), 'GetProcAddress');
  LoadLibraryInfo.Result := 0;

  LoadLibraryInfo.Path := Process.NewWideString(Path);

  if InitializationName = nil then
    LoadLibraryInfo.InitializationName := nil
  else
    LoadLibraryInfo.InitializationName := Process.NewAnsiString(InitializationName);

  Extern := Process.New(LoadLibraryInfo, SizeOf(TLoadLibraryInfo));

  Result := Process.CopyOnRun(@ExternLoadLibrary, $1000, Extern);

  if Result <> 0 then
    raise Exception.Create(GetErrorMessage(Result));

  Process.Read(Extern, Result, 4);
end;


end.
