unit LaunchSettings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Generics.Collections, Sli, ImgList, ActnList,
  Menus;

type
  TSettingsForm = class(TForm)
    ButtonClose: TButton;
    GroupBox1: TGroupBox;
    TriggerDebug: TCheckBox;
    ShowFPS: TCheckBox;
    CaptureMouse: TCheckBox;
    DisableAltTab: TCheckBox;
    DisableWinKey: TCheckBox;
    DisableTextures: TCheckBox;
    Label1: TLabel;
    SCVersion: TComboBox;
    GroupBox2: TGroupBox;
    PluginListView: TListView;
    ImageList: TImageList;
    ButtonEnable: TButton;
    ButtonDisable: TButton;
    ActionList: TActionList;
    PluginEnable: TAction;
    PluginDisable: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PluginEnableUpdate(Sender: TObject);
    procedure PluginDisableUpdate(Sender: TObject);
    procedure PluginDisableExecute(Sender: TObject);
    procedure PluginEnableExecute(Sender: TObject);
    procedure PluginListViewCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
  private
    { Private declarations }
  public
    procedure Open;
  end;

var
  SettingsForm: TSettingsForm;

implementation

uses GameSetup, Plugins;

{$R *.dfm}

procedure TSettingsForm.FormCreate(Sender: TObject);
var
  SearchRec: TSearchRec;
  List: TList;
  ListItem: TListItem;
  i: Integer;
begin
  SystemFont(Font);

  SetupPlugins(SearchPath + 'Plugins\');

  for i := 0 to PluginList.Count - 1 do
    with PluginList[i] do
      begin
        ListItem := PluginListView.Items.Add;
        ListItem.Caption := Name;
        ListItem.SubItems.Add(Version);
        ListItem.SubItems.Add(Author);

        if Enabled then
          ListItem.ImageIndex := 0
        else
          ListItem.ImageIndex := 1;

        ListItem.Data := PluginList[i];
      end;

  List := TList.Create;
  try
    try
      if FindFirst(GamePath + 'Versions\Base*', faAnyFile, SearchRec) = 0 then
        repeat
          if (SearchRec.Name <> '.') and (SearchRec.Name <> '..') and ((SearchRec.Attr and faDirectory) <> 0) then
            List.Add(Pointer(StrToInt(Copy(SearchRec.Name, 5, Length(SearchRec.Name)))));
        until FindNext(SearchRec) <> 0;
    finally
      FindClose(SearchRec);
    end;

    List.SortList(function (Item1, Item2: Pointer): Integer
      begin
        Result := Integer(Item2) - Integer(Item1);
      end);

    for i := 0 to List.Count - 1 do
      SCVersion.Items.AddObject('Build ' + IntToStr(Integer(List[i])), TObject(List[i]));

    if List.Count = 0 then
      begin
        MessageDlg('Can''t find the StarCraft II executable in your installation!', mtError, [mbOK], 0);
        Application.Terminate;
      end;

    SCVersion.ItemIndex := 0;

  finally
    List.Free;
  end;
end;

procedure TSettingsForm.FormDestroy(Sender: TObject);
begin
  FreePlugins;
end;

procedure TSettingsForm.Open;
var
  i, Version: Integer;
begin
  TriggerDebug.Checked := Settings.ReadBool('SC2Launcher', 'TriggerDebug', False);
  ShowFPS.Checked := Settings.ReadBool('SC2Launcher', 'ShowFPS', False);
  CaptureMouse.Checked := Settings.ReadBool('SC2Launcher', 'CaptureMouse', False);
  DisableAltTab.Checked := Settings.ReadBool('SC2Launcher', 'DisableAltTab', False);
  DisableWinKey.Checked := Settings.ReadBool('SC2Launcher', 'DisableWinKey', False);
  DisableTextures.Checked := Settings.ReadBool('SC2Launcher', 'DisableTextures', False);
  Version := Settings.ReadInteger('SC2Launcher', 'Version', 0);

  for i := 0 to SCVersion.Items.Count - 1 do
    if Version = Integer(SCVersion.Items.Objects[i]) then
      begin
        SCVersion.ItemIndex := i;
        Break;
      end;

  ShowModal;

  Settings.WriteBool('SC2Launcher', 'TriggerDebug', TriggerDebug.Checked);
  Settings.WriteBool('SC2Launcher', 'ShowFPS', ShowFPS.Checked);
  Settings.WriteBool('SC2Launcher', 'CaptureMouse', CaptureMouse.Checked);
  Settings.WriteBool('SC2Launcher', 'DisableAltTab', DisableAltTab.Checked);
  Settings.WriteBool('SC2Launcher', 'DisableWinKey', DisableWinKey.Checked);
  Settings.WriteBool('SC2Launcher', 'DisableTextures', DisableTextures.Checked);
  Settings.WriteInteger('SC2Launcher', 'Version', Integer(SCVersion.Items.Objects[SCVersion.ItemIndex]));

  for i := 0 to PluginList.Count - 1 do
    Settings.WriteBool('Plugins', ExtractFileName(PluginList[i].Path), PluginList[i].Enabled);

end;

procedure TSettingsForm.PluginDisableExecute(Sender: TObject);
begin
  if PluginListView.Selected <> nil then
    begin
      PluginListView.Selected.ImageIndex := 1;
      TPlugin(PluginListView.Selected.Data).Disable;
    end;
end;

procedure TSettingsForm.PluginDisableUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := (PluginListView.Selected <> nil) and TPlugin(PluginListView.Selected.Data).Enabled;
end;

procedure TSettingsForm.PluginEnableExecute(Sender: TObject);
begin
  if PluginListView.Selected <> nil then
    begin
      PluginListView.Selected.ImageIndex := 0;
      TPlugin(PluginListView.Selected.Data).Enable(True);
    end;
end;

procedure TSettingsForm.PluginEnableUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := (PluginListView.Selected <> nil) and (not TPlugin(PluginListView.Selected.Data).Enabled);
end;

procedure TSettingsForm.PluginListViewCustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
begin
  if TPlugin(Item.Data).Enabled then
    Sender.Canvas.Font.Color := clWindowText
  else
    Sender.Canvas.Font.Color := clGrayText;
end;

end.
