program SC2Launcher;

uses
  Windows,
  SysUtils,
  Forms,
  Dialogs,
  Registry,
  IniFiles,
  FileCtrl,
  GameSetup in 'GameSetup.pas' {GameForm},
  MapSelector in 'MapSelector.pas' {MapForm},
  MapUpdater in 'MapUpdater.pas',
  MpqAPI in 'MpqAPI.pas',
  LaunchSettings in 'LaunchSettings.pas' {SettingsForm},
  StreamUtils in 'StreamUtils.pas',
  Process in 'Process.pas',
  Launcher in 'Launcher.pas',
  Injector in 'Injector.pas',
  Plugins in 'Plugins.pas',
  Sli in 'Sli.pas';

{$R *.res}

function ValidPath(Path: String): Boolean;
begin
  Path := IncludeTrailingPathDelimiter(Path);
  Result := DirectoryExists(Path) and FileExists(Path + 'StarCraft II.exe');
end;

begin
  Settings := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    with TRegistry.Create do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        if OpenKeyReadOnly('SOFTWARE\Wow6432Node\Blizzard Entertainment\StarCraft II') then
          GamePath := ReadString('InstallPath')
        else if OpenKeyReadOnly('SOFTWARE\Blizzard Entertainment\StarCraft II') then
          GamePath := ReadString('InstallPath');
      finally
        Free;
      end;

    GamePath := Settings.ReadString('SC2Launcher', 'GamePath', GamePath);

    while True do
      begin
        if ValidPath(GamePath) then
          begin
            Settings.WriteString('SC2Launcher', 'GamePath', GamePath);
            Break;
          end
        else if GamePath <> '' then
          MessageDlg('The folder ''' + GamePath + ''' doesn''t seem to be a valid StarCraft II installation. Please choose another folder.', mtError, [mbOK], 0);

        if not SelectDirectory('Please select your StarCraft II folder:', '', GamePath, [sdNewUI], nil) then
          Exit;
      end;

    GamePath := IncludeTrailingPathDelimiter(GamePath);

    ReportMemoryLeaksOnShutdown := True;
    Application.Initialize;
    Application.MainFormOnTaskbar := True;
    Application.Title := 'StarCraft II Launcher';
    Application.CreateForm(TGameForm, GameForm);
  Application.CreateForm(TMapForm, MapForm);
  Application.CreateForm(TSettingsForm, SettingsForm);
  Application.Run;
  finally
    Settings.Free;
  end;
end.
