unit StreamUtils;

interface

uses Classes;

function SkipNullString(Stream: TStream): Cardinal;
procedure StreamInsert(Stream: TStream; const Data; Size: Cardinal);
procedure StreamRemove(Stream: TStream; Size: Cardinal);

implementation

function SkipNullString(Stream: TStream): Cardinal;
var C: AnsiChar;
begin
  Result := 0;
  Stream.Read(C, 1);

  while C <> #0 do
    begin
      Stream.Read(C, 1);
      Inc(Result);
    end;
end;

procedure StreamInsert(Stream: TStream; const Data; Size: Cardinal);
var
  MemoryStream: TMemoryStream;
  Position: Int64;
begin
  if Size = 0 then
    Exit;

  MemoryStream := TMemoryStream.Create;
  try
    MemoryStream.Size := Stream.Size + Size;
    Position := Stream.Position;

    Stream.Position := 0;
    Stream.Read(MemoryStream.Memory^, Position);

    MemoryStream.Position := Position;
    MemoryStream.Write(Data, Size);

    Stream.Read(Pointer(Cardinal(MemoryStream.Memory) + Position + Size)^, Stream.Size - Position);
    Stream.Position := 0;
    Stream.Size := 0;

    MemoryStream.SaveToStream(Stream);

    Stream.Position := Position + Size;
  finally
    MemoryStream.Free;
  end;
end;

procedure StreamRemove(Stream: TStream; Size: Cardinal);
var
  MemoryStream: TMemoryStream;
  Position: Int64;
begin
  if Size = 0 then
    Exit;

  MemoryStream := TMemoryStream.Create;
  try
    MemoryStream.Size := Stream.Size - Size;
    Position := Stream.Position;

    Stream.Position := 0;
    Stream.Read(MemoryStream.Memory^, Position);

    Stream.Seek(Size, soFromCurrent); // Skip unwanted data

    Stream.Read(Pointer(Cardinal(MemoryStream.Memory) + Position)^, Stream.Size - Position - Size);

    Stream.Position := 0;
    Stream.Size := 0;

    MemoryStream.SaveToStream(Stream);

    Stream.Position := Position;
  finally
    MemoryStream.Free;
  end;
end;

end.
